package com.ruoyi.cxxm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.cxxm.domain.Task;
import com.ruoyi.cxxm.domain.TaskCheckImage;
import com.ruoyi.cxxm.domain.TaskCheckVideo;
import com.ruoyi.cxxm.domain.dto.TaskCheckSubmit;
import com.ruoyi.cxxm.domain.enums.RoleName;
import com.ruoyi.cxxm.domain.enums.SFSB;
import com.ruoyi.cxxm.domain.enums.XCJB;
import com.ruoyi.cxxm.mapper.TaskCheckMapper;
import com.ruoyi.cxxm.domain.TaskCheck;
import com.ruoyi.cxxm.mapper.TaskMapper;
import com.ruoyi.cxxm.service.ITaskCheckImageService;
import com.ruoyi.cxxm.service.ITaskCheckService;
import com.ruoyi.cxxm.service.ITaskCheckVideoService;
import com.ruoyi.cxxm.service.ITaskService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.Objects;


/**
 * 任务巡查记录Service业务层处理
 *
 * @author ruoyi
 * @date 2024-03-28
 */
@Service
public class TaskCheckServiceImpl extends ServiceImpl<TaskCheckMapper, TaskCheck> implements ITaskCheckService {

    @Autowired
    private ITaskCheckService taskCheckService;
    @Autowired
    private ITaskService taskService;
    @Autowired
    private ITaskCheckImageService taskCheckImageService;
    @Autowired
    private ITaskCheckVideoService taskCheckVideoService;
    @Autowired
    private TaskMapper taskMapper;

    /**
     * 保存任务提交记录和文件
     *
     * @param taskCheckSubmit
     * @param files
     * @return
     * @throws IOException
     */
    @Override
    @Transactional
    public boolean saveTaskCheckSubmitAndFiles(TaskCheckSubmit taskCheckSubmit, List<MultipartFile> files) throws IOException {
        // 填报数据处理
        Long userId = SecurityUtils.getUserId();
        taskCheckSubmit.setXcryId(userId);
        SysUser user = SecurityUtils.getLoginUser().getUser();
        List<SysRole> roles = user.getRoles();
        for (SysRole role : roles) {
            if (((Objects.equals(role.getRoleName(), RoleName.XJGLY.getValue()))
                    || Objects.equals(role.getRoleName(), RoleName.XJDCY.getValue()))
            ) {
                if (!Objects.equals(taskCheckSubmit.getXcjb(), XCJB.ZJ.getValue())) {
                    taskCheckSubmit.setXcjb(XCJB.XJ.getValue());
                }

            }
            if ((Objects.equals(role.getRoleName(), RoleName.ZJGLY.getValue()))
                    || Objects.equals(role.getRoleName(), RoleName.ZJDCY.getValue())) {
                taskCheckSubmit.setXcjb(XCJB.ZJ.getValue());
            }
        }
        // 查询任务
        Long taskId = taskCheckSubmit.getTaskId();
        Task task = taskService.getById(taskId);
        if (task == null) {
            throw new ServiceException(StringUtils.format("id={}的任务不存在!", taskId));
        }
        // 填报数据
        LambdaQueryWrapper<TaskCheck> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaskCheck::getTaskId, taskId);
        queryWrapper.eq(TaskCheck::getXcryId, userId);
        queryWrapper.eq(TaskCheck::getSfsb, SFSB.WSB.getValue());
        queryWrapper.last("limit 1");
        TaskCheck taskCheck = taskCheckService.getOne(queryWrapper);
        if (taskCheck == null) {
            taskCheckService.saveOrUpdate(taskCheckSubmit);
            taskCheck = taskCheckService.getOne(queryWrapper);

        }
        Long tcId = taskCheck.getId();
        taskCheckSubmit.setId(tcId);
        taskCheckService.saveOrUpdate(taskCheckSubmit);
        // 保存图片列表
        List<TaskCheckImage> taskCheckImages = taskCheckSubmit.getTaskCheckImages();
        for (TaskCheckImage taskCheckImage : taskCheckImages) {
            for (MultipartFile file : files) {
                if (Objects.nonNull(taskCheckImage.getOriginFileName()) && Objects.equals(taskCheckImage.getOriginFileName(), file.getOriginalFilename())) {
                    String cclj = FileUploadUtils.upload(RuoYiConfig.getUploadPath(), file);
                    taskCheckImage.setCclj(cclj);
                }
            }
            taskCheckImage.setTcId(tcId);
        }
        taskCheckImageService.saveOrUpdateBatch(taskCheckSubmit.getTaskCheckImages());
        // 保存视频列表
        List<TaskCheckVideo> taskCheckVideos = taskCheckSubmit.getTaskCheckVideos();
        for (TaskCheckVideo taskCheckVideo : taskCheckVideos) {
            for (MultipartFile file : files) {
                if (Objects.nonNull(taskCheckVideo.getOriginFileName()) && Objects.equals(taskCheckVideo.getOriginFileName(), file.getOriginalFilename())) {
                    String cclj = FileUploadUtils.upload(RuoYiConfig.getUploadPath(), file);
                    taskCheckVideo.setCclj(cclj);
                }
            }
            taskCheckVideo.setTcId(tcId);
        }
        taskCheckVideoService.saveOrUpdateBatch(taskCheckSubmit.getTaskCheckVideos());
        return true;
    }


    /**
     * 上报填报记录
     *
     * @param tcId
     * @return
     */
    @Override
    @Transactional
    public boolean reportTaskCheckInfo(Long tcId) {
        TaskCheck taskCheck = taskCheckService.getById(tcId);
        if (Objects.equals(taskCheck.getSfsb(), SFSB.YSB.getValue())) {
            return true;
        }
        // 更新任务巡查表
        taskCheck.setId(tcId);
        taskCheck.setSfsb(SFSB.YSB.getValue());
        taskCheck.setSbsj(DateUtils.getNowDate());
        taskCheckService.saveOrUpdate(taskCheck);
        // 更新任务表
        Task task = taskService.getById(taskCheck.getTaskId());
        LambdaQueryWrapper<TaskCheck> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaskCheck::getTaskId, taskCheck.getTaskId());
        queryWrapper.eq(TaskCheck::getSfsb, SFSB.YSB.getValue());
        queryWrapper.orderByDesc(TaskCheck::getXcsj);
        queryWrapper.last("limit 1");
        TaskCheck lastTaskCheck = taskCheckService.getOne(queryWrapper);
        if (Objects.nonNull(lastTaskCheck) && taskCheck.getXcsj().compareTo(lastTaskCheck.getXcsj()) >= 0) {
            task.setTcId(tcId);
            task.setXcsj(lastTaskCheck.getXcsj());
            task.setXcryId(SecurityUtils.getUserId());
            // 修改任务表巡查次数
            task.setYxccs(task.getYxccs() + 1); // 已巡查次数+1
            SysUser user = SecurityUtils.getLoginUser().getUser();
            List<SysRole> roles = user.getRoles();
            for (SysRole role : roles) {
                if (((Objects.equals(role.getRoleName(), RoleName.XJGLY.getValue()))
                        || Objects.equals(role.getRoleName(), RoleName.XJDCY.getValue()))
                ) {
                    task.setZjxccs(task.getZjxccs() + 1); // 州级巡查次数+1
                }
                if ((Objects.equals(role.getRoleName(), RoleName.ZJGLY.getValue()))
                        || Objects.equals(role.getRoleName(), RoleName.ZJDCY.getValue())) {
                    task.setZjxccs(task.getZjxccs() + 1); // 县级巡查次数+1
                }
            }
            taskService.saveOrUpdate(task);
        }
        return true;
    }

    /**
     * 取消上报
     *
     * @param tcId
     * @return
     */
    @Override
    @Transactional
    public boolean cancelReportTaskCheckInfo(Long tcId) {
        TaskCheck taskCheck = taskCheckService.getById(tcId);
        if (Objects.equals(taskCheck.getSfsb(), SFSB.WSB.getValue())) {
            return true;
        }
        // 更新任务巡查表
        taskCheck.setId(tcId);
        taskCheck.setSfsb(SFSB.WSB.getValue());
        taskCheck.setSbsj(DateUtils.getNowDate());
        taskCheckService.saveOrUpdate(taskCheck);
        // 更新任务表
        Task task = taskService.getById(taskCheck.getTaskId());
        LambdaQueryWrapper<TaskCheck> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaskCheck::getTaskId, taskCheck.getTaskId());
        queryWrapper.eq(TaskCheck::getSfsb, SFSB.YSB.getValue());
        queryWrapper.orderByDesc(TaskCheck::getXcsj);
        queryWrapper.last("limit 1");
        TaskCheck lastTaskCheck = taskCheckService.getOne(queryWrapper);
        // 修改任务表巡查次数
        task.setYxccs(task.getYxccs() - 1); // 已巡查次数-1
        SysUser user = SecurityUtils.getLoginUser().getUser();
        List<SysRole> roles = user.getRoles();
        for (SysRole role : roles) {
            if (((Objects.equals(role.getRoleName(), RoleName.XJGLY.getValue()))
                    || Objects.equals(role.getRoleName(), RoleName.XJDCY.getValue()))
            ) {
                task.setZjxccs(task.getZjxccs() - 1); // 州级巡查次数-1
            }
            if ((Objects.equals(role.getRoleName(), RoleName.ZJGLY.getValue()))
                    || Objects.equals(role.getRoleName(), RoleName.ZJDCY.getValue())) {
                task.setZjxccs(task.getZjxccs() - 1); // 县级巡查次数-1
            }
        }
        if (Objects.nonNull(lastTaskCheck)) {
            task.setTcId(lastTaskCheck.getId());
            task.setXcsj(lastTaskCheck.getXcsj());
            task.setXcryId(lastTaskCheck.getXcryId());
        } else {
            taskMapper.setLastTcInfoFieldToNullById(taskCheck.getTaskId());
        }
        taskService.saveOrUpdate(task);
        return true;
    }
}

