package com.ruoyi.cxxm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.cxxm.domain.Tbxx;
import com.ruoyi.cxxm.mapper.TaskMapper;
import com.ruoyi.cxxm.domain.Task;
import com.ruoyi.cxxm.mapper.TbxxMapper;
import com.ruoyi.cxxm.service.ITaskService;
import com.ruoyi.system.mapper.SysDeptMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;


/**
 * 任务Service业务层处理
 *
 * @author ruoyi
 * @date 2024-03-28
 */
@Service
public class TaskServiceImpl extends ServiceImpl<TaskMapper, Task> implements ITaskService {

    @Autowired
    private SysDeptMapper deptMapper;

    @Autowired
    private TbxxMapper tbxxMapper;

    @Autowired
    private TaskMapper taskMapper;


    /**
     * 导入任务
     * @param tbxx
     * @return
     */
    @Override
    @Transactional
    public int importTaskByTbxx(Tbxx tbxx,String rwlx) {
        // 新建任务信息
        Task task = new Task();
        task.setRwlx(rwlx);
        task.setTbbh(tbxx.getTbbh());
        task.setTbszx(tbxx.getXian());
        task.setDrryId(SecurityUtils.getUserId());
        task.setDrsj(DateUtils.getNowDate());
        // 获取图斑所在县的部门id
        String xian = tbxx.getXian();
        SysDept dept = deptMapper.selectDeptByDeptName(xian);
        if (dept != null) {
            task.setDeptId(dept.getDeptId());
            tbxx.setDeptId(dept.getDeptId());
        }
        // 插入图斑信息
        tbxx.setCreateBy(SecurityUtils.getUsername());
        tbxx.setCreateTime(DateUtils.getNowDate());
        tbxxMapper.insertTbxx(tbxx);
        // 插入任务
        return taskMapper.insert(task);
    }

    /**
     * 根据taskIds删除任务和图斑信息
     * @param taskIds
     * @return
     */
    @Override
    @Transactional
    public int removeTaskAndTbxxByTaskIds(List<Long> taskIds) {
        // 删除图斑信息
        for (Long taskId : taskIds) {
            Task task = taskMapper.selectById(taskId);
            LambdaQueryWrapper<Tbxx> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(Tbxx::getTbbh,task.getTbbh());
            tbxxMapper.delete(queryWrapper);
        }
        // 删除任务信息
        return taskMapper.deleteBatchIds(taskIds);
    }
}

