package com.ruoyi.web.controller.cxxm;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.PageDomain;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.cxxm.domain.TaskCheckAudit;
import com.ruoyi.cxxm.service.ITaskCheckAuditService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 任务巡查审核Controller
 *
 * @author ruoyi
 * @date 2024-03-28
 */
@RestController
@RequestMapping("/cxxm/taskCheckAudit")
public class TaskCheckAuditController extends BaseController
{
    @Autowired
    private ITaskCheckAuditService taskCheckAuditService;

/**
 * 查询任务巡查审核列表
 */
@PreAuthorize("@ss.hasPermi('cxxm:taskCheckAudit:list')")
@GetMapping("/list")
    public TableDataInfo list(TaskCheckAudit taskCheckAudit, PageDomain pageDomain)
    {
        startPage();
        List<TaskCheckAudit> list = taskCheckAuditService.list(new QueryWrapper<>(taskCheckAudit));
        return getDataTable(list);
    }

    /**
     * 导出任务巡查审核列表
     */
    @PreAuthorize("@ss.hasPermi('cxxm:taskCheckAudit:export')")
    @Log(title = "任务巡查审核", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TaskCheckAudit taskCheckAudit)
    {
        List<TaskCheckAudit> list = taskCheckAuditService.list(new QueryWrapper<>(taskCheckAudit));
        ExcelUtil<TaskCheckAudit> util = new ExcelUtil<TaskCheckAudit>(TaskCheckAudit.class);
        util.exportExcel(response, list, "任务巡查审核数据");
    }

    /**
     * 获取任务巡查审核详细信息
     */
    @PreAuthorize("@ss.hasPermi('cxxm:taskCheckAudit:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(taskCheckAuditService.getById(id));
    }

    /**
     * 新增任务巡查审核
     */
    @PreAuthorize("@ss.hasPermi('cxxm:taskCheckAudit:add')")
    @Log(title = "任务巡查审核", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TaskCheckAudit taskCheckAudit)
    {
        taskCheckAudit.setShsj(DateUtils.getNowDate());
        taskCheckAudit.setShryId(SecurityUtils.getUserId());
        return toAjax(taskCheckAuditService.saveOrUpdate(taskCheckAudit));
    }

    /**
     * 修改任务巡查审核
     */
    @PreAuthorize("@ss.hasPermi('cxxm:taskCheckAudit:edit')")
    @Log(title = "任务巡查审核", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TaskCheckAudit taskCheckAudit)
    {
        taskCheckAudit.setShsj(DateUtils.getNowDate());
        taskCheckAudit.setShryId(SecurityUtils.getUserId());
        return toAjax(taskCheckAuditService.saveOrUpdate(taskCheckAudit));
    }

    /**
     * 删除任务巡查审核
     */
    @PreAuthorize("@ss.hasPermi('cxxm:taskCheckAudit:remove')")
    @Log(title = "任务巡查审核", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids)
    {
        return toAjax(taskCheckAuditService.removeByIds(ids));
    }
}

