package com.ruoyi.web.controller.cxxm;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.cxxm.domain.*;
import com.ruoyi.cxxm.domain.enums.SFSB;
import com.ruoyi.cxxm.domain.dto.TaskCheckSubmit;
import com.ruoyi.cxxm.domain.vo.TaskCheckLog;
import com.ruoyi.cxxm.domain.vo.TaskCheckSubmitVo;
import com.ruoyi.cxxm.service.*;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.PageDomain;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

/**
 * 任务巡查记录Controller
 *
 * @author ruoyi
 * @date 2024-03-28
 */
@RestController
@RequestMapping("/cxxm/taskCheck")
public class TaskCheckController extends BaseController {
    @Autowired
    private ITaskCheckService taskCheckService;
    @Autowired
    private ITaskService taskService;
    @Autowired
    private ITaskCheckImageService taskCheckImageService;
    @Autowired
    private ITaskCheckVideoService taskCheckVideoService;
    @Autowired
    private ITbxxService tbxxService;
    @Autowired
    private ITaskCheckAuditService taskCheckAuditService;

    /**
     * 查询任务巡查记录列表
     */
    @PreAuthorize("@ss.hasPermi('cxxm:taskCheck:list')")
    @GetMapping("/list")
    public TableDataInfo list(TaskCheck taskCheck, PageDomain pageDomain) {
        startPage();
        List<TaskCheck> list = taskCheckService.list(new QueryWrapper<>(taskCheck));
        return getDataTable(list);
    }

    /**
     * 导出任务巡查记录列表
     */
    @PreAuthorize("@ss.hasPermi('cxxm:taskCheck:export')")
    @Log(title = "任务巡查记录导出", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TaskCheck taskCheck) {
        List<TaskCheck> list = taskCheckService.list(new QueryWrapper<>(taskCheck));
        ExcelUtil<TaskCheck> util = new ExcelUtil<TaskCheck>(TaskCheck.class);
        util.exportExcel(response, list, "任务巡查记录数据");
    }

    /**
     * 根据taskId获取任务巡查记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('cxxm:taskCheck:query')")
    @GetMapping(value = "/{taskId}")
    public AjaxResult getInfoByTaskId(@PathVariable("taskId") Long taskId) {
        TaskCheckSubmitVo taskCheckSubmitVo = new TaskCheckSubmitVo();
        // 最后一条填报内容
        LambdaQueryWrapper<TaskCheck> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaskCheck::getTaskId, taskId);
        queryWrapper.eq(TaskCheck::getSfsb, SFSB.YSB.getValue());
        queryWrapper.orderByDesc(TaskCheck::getId);
        queryWrapper.last("limit 1");
        TaskCheck taskCheck = taskCheckService.getOne(queryWrapper);
        if (taskCheck == null) {
            return AjaxResult.success();
        }
        BeanUtils.copyProperties(taskCheck, taskCheckSubmitVo);
        // 图片
        LambdaQueryWrapper<TaskCheckImage> tcImagesQueryWrapper = new LambdaQueryWrapper<>();
        tcImagesQueryWrapper.eq(TaskCheckImage::getTcId, taskCheck.getId());
        List<TaskCheckImage> taskCheckImages = taskCheckImageService.list(tcImagesQueryWrapper);
        taskCheckSubmitVo.setTaskCheckImages(taskCheckImages);
        // 视频
        LambdaQueryWrapper<TaskCheckVideo> tcVideosQueryWrapper = new LambdaQueryWrapper<>();
        tcVideosQueryWrapper.eq(TaskCheckVideo::getTcId, taskCheck.getId());
        List<TaskCheckVideo> taskCheckVideos = taskCheckVideoService.list(tcVideosQueryWrapper);
        taskCheckSubmitVo.setTaskCheckVideos(taskCheckVideos);
        // 图斑信息
        Task task = taskService.getById(taskCheck.getTaskId());
        Tbxx tbxx = tbxxService.selectTbxxByTbbh(task.getTbbh());
        taskCheckSubmitVo.setTbxx(tbxx);
        // 填报记录列表
        LambdaQueryWrapper<TaskCheck> tcLogQueryWrapper = new LambdaQueryWrapper<>();
        tcLogQueryWrapper.eq(TaskCheck::getTaskId, taskId);
        tcLogQueryWrapper.orderByDesc(TaskCheck::getId);
        List<TaskCheck> tcList = taskCheckService.list(tcLogQueryWrapper);
        List<TaskCheckLog> tcLogList = new ArrayList<>();
        for (TaskCheck item : tcList) {
            // 跳过未上报,且非本人巡查的记录
            if (Objects.equals(item.getSfsb(), SFSB.WSB.getValue()) && !Objects.equals(item.getXcryId(), SecurityUtils.getUserId())) {
                continue;
            }
            TaskCheckLog taskCheckLog = new TaskCheckLog();
            BeanUtils.copyProperties(item, taskCheckLog);
            taskCheckLog.setTcId(item.getId());
            // 填报记录图片
            LambdaQueryWrapper<TaskCheckImage> tcLogImagesQueryWrapper = new LambdaQueryWrapper<>();
            tcLogImagesQueryWrapper.eq(TaskCheckImage::getTcId, item.getId());
            List<TaskCheckImage> tcLogImages = taskCheckImageService.list(tcLogImagesQueryWrapper);
            taskCheckLog.setTaskCheckImages(tcLogImages);
            // 填报记录视频
            LambdaQueryWrapper<TaskCheckVideo> tcLogVideosQueryWrapper = new LambdaQueryWrapper<>();
            tcLogVideosQueryWrapper.eq(TaskCheckVideo::getTcId, item.getId());
            List<TaskCheckVideo> tcLogVideos = taskCheckVideoService.list(tcLogVideosQueryWrapper);
            taskCheckLog.setTaskCheckVideos(tcLogVideos);
            // 填报记录审核
            LambdaQueryWrapper<TaskCheckAudit> tcAuditQueryWrapper = new LambdaQueryWrapper<>();
            tcAuditQueryWrapper.eq(TaskCheckAudit::getTcId, item.getId());
            TaskCheckAudit taskCheckAudit = taskCheckAuditService.getOne(tcAuditQueryWrapper);
            taskCheckLog.setTaskCheckAudit(taskCheckAudit);
            tcLogList.add(taskCheckLog);
        }
        taskCheckSubmitVo.setTaskCheckLogs(tcLogList);
        return AjaxResult.success(taskCheckSubmitVo);
    }

    /**
     * 新增任务巡查记录
     */
    @PreAuthorize("@ss.hasPermi('cxxm:taskCheck:add')")
    @Log(title = "任务巡查记录填报", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(MultipartHttpServletRequest request, List<MultipartFile> files) throws IOException {
        String taskCheckSubmitStr = request.getParameter("taskCheckSubmit");
        TaskCheckSubmit taskCheckSubmit = JSONObject.parseObject(taskCheckSubmitStr, TaskCheckSubmit.class);
        taskCheckService.saveTaskCheckSubmitAndFiles(taskCheckSubmit, files);
        return AjaxResult.success();
    }

    /**
     * 修改任务巡查记录
     */
    @PreAuthorize("@ss.hasPermi('cxxm:taskCheck:edit')")
    @Log(title = "任务巡查记录修改", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(MultipartHttpServletRequest request, List<MultipartFile> files) throws IOException {
        String taskCheckSubmitStr = request.getParameter("taskCheckSubmit");
        TaskCheckSubmit taskCheckSubmit = JSONObject.parseObject(taskCheckSubmitStr, TaskCheckSubmit.class);
        taskCheckService.saveTaskCheckSubmitAndFiles(taskCheckSubmit, files);
        return AjaxResult.success();
    }

    /**
     * 删除任务巡查记录
     */
    @PreAuthorize("@ss.hasPermi('cxxm:taskCheck:remove')")
    @Log(title = "任务巡查记录删除", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(taskCheckService.removeByIds(ids));
    }

    /**
     * 上报任务巡查记录
     */
    @PreAuthorize("@ss.hasPermi('cxxm:taskCheck:edit')")
    @Log(title = "任务巡查记录上报", businessType = BusinessType.UPDATE)
    @PutMapping("/report/{tcId}")
    public AjaxResult report(@PathVariable Long tcId) {
        return toAjax(taskCheckService.reportTaskCheckInfo(tcId));
    }

    /**
     * 取消上报任务巡查记录
     */
    @PreAuthorize("@ss.hasPermi('cxxm:taskCheck:edit')")
    @Log(title = "任务巡查记录上报", businessType = BusinessType.UPDATE)
    @PutMapping("/cancelReport/{tcId}")
    public AjaxResult cancelReport(@PathVariable Long tcId) {
        return toAjax(taskCheckService.cancelReportTaskCheckInfo(tcId));
    }
}

