package com.ruoyi.web.controller.cxxm;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.cxxm.domain.Tbxx;
import com.ruoyi.cxxm.domain.enums.XFQK;
import com.ruoyi.cxxm.domain.query.TaskQuery;
import com.ruoyi.cxxm.domain.vo.TaskDetailVo;
import com.ruoyi.cxxm.mapper.DeptMapper;
import com.ruoyi.cxxm.service.ITbxxService;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.PageDomain;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.cxxm.domain.Task;
import com.ruoyi.cxxm.service.ITaskService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 任务Controller
 *
 * @author ruoyi
 * @date 2024-03-28
 */
@RestController
@RequestMapping("/cxxm/task")
public class TaskController extends BaseController {
    @Autowired
    private ITaskService taskService;
    @Autowired
    private ITbxxService tbxxService;
    @Autowired
    private DeptMapper deptMapper;

    /**
     * 查询任务列表
     */
    @PreAuthorize("@ss.hasPermi('cxxm:task:list')")
    @GetMapping("/list")
    public TableDataInfo list(TaskQuery task, PageDomain pageDomain) {
        startPage();
        LambdaQueryWrapper<Task> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(Objects.nonNull(task.getTbbh()), Task::getTbbh, task.getTbbh());
        queryWrapper.eq(Objects.nonNull(task.getXfqk()), Task::getXfqk, task.getXfqk());
        queryWrapper.eq(Objects.nonNull(task.getRwlx()), Task::getRwlx, task.getRwlx());
        queryWrapper.in(Objects.nonNull(task.getDeptId()), Task::getDeptId, deptMapper.getAllAncestorsByDeptId(task.getDeptId()));
        List<String> drsjQuery = task.getDrsjQuery();
        if (drsjQuery != null && drsjQuery.size() == 2) {
            queryWrapper.apply(StringUtils.format("DATE_FORMAT(drsj, '%Y-%m') between '{}' and '{}'",
                    drsjQuery.get(0), drsjQuery.get(1)));
        }
        queryWrapper.orderByAsc(Task::getXfqk);
        queryWrapper.orderByDesc(Task::getXfsj);
        queryWrapper.orderByDesc(Task::getDrsj);
        List<Task> list = taskService.list(queryWrapper);
        return getDataTable(list);
    }

    /**
     * 导出任务列表
     */
    @PreAuthorize("@ss.hasPermi('cxxm:task:export')")
    @Log(title = "任务", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, Task task) {
        List<Task> list = taskService.list(new QueryWrapper<>(task));
        ExcelUtil<Task> util = new ExcelUtil<Task>(Task.class);
        util.exportExcel(response, list, "任务数据");
    }

    /**
     * 获取任务详细信息
     */
    @PreAuthorize("@ss.hasPermi('cxxm:task:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        Task task = taskService.getById(id);
        Tbxx tbxx = tbxxService.selectTbxxByTbbh(task.getTbbh());
        TaskDetailVo taskDetailVo = new TaskDetailVo();
        BeanUtils.copyProperties(task, taskDetailVo);
        taskDetailVo.setTbxx(tbxx);
        return AjaxResult.success(taskDetailVo);
    }

    /**
     * 新增任务
     */
    @PreAuthorize("@ss.hasPermi('cxxm:task:add')")
    @Log(title = "任务", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody Task task) {
        return toAjax(taskService.save(task));
    }

    /**
     * 修改任务
     */
    @PreAuthorize("@ss.hasPermi('cxxm:task:edit')")
    @Log(title = "任务", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody Task task) {
        return toAjax(taskService.updateById(task));
    }

    /**
     * 删除任务
     */
    @PreAuthorize("@ss.hasPermi('cxxm:task:remove')")
    @Log(title = "任务", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable List<Long> ids) {
        return toAjax(taskService.removeTaskAndTbxxByTaskIds(ids));
    }

    /**
     * 导入任务
     *
     * @param file
     * @param rwlx
     * @return
     * @throws Exception
     */
    @Log(title = "导入任务", businessType = BusinessType.IMPORT)
    @PreAuthorize("@ss.hasPermi('cxxm:task:import')")
    @PostMapping("/importData")
    public AjaxResult importData(MultipartFile file, String rwlx) throws Exception {
        try (InputStream inputStream = file.getInputStream()) {
            // 读取数据
            BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));
            StringBuilder content = new StringBuilder();
            String line;
            while ((line = reader.readLine()) != null) {
                content.append(line);
            }
            // 解析数据
            Map map = JSONObject.parseObject(content.toString(), Map.class);
            Object features = map.get("features");
            List importData = JSONObject.parseObject(features.toString(), List.class);
            List<String> failTbbhList = new ArrayList<>();
            for (Object data : importData) {
                Map item = JSONObject.parseObject(data.toString(), Map.class);
                Tbxx tbxx = new Tbxx();
                // 图斑空间数据
                Object geometry = item.get("geometry");
                tbxx.setGis(geometry.toString());
                // 图斑信息
                Object properties = item.get("properties");
                Map tbxxMap = JSONObject.parseObject(properties.toString(), Map.class);
                tbxx.setTbbh(tbxxMap.get("KCTBBH") != null ? tbxxMap.get("KCTBBH").toString() : "");
                tbxx.setKsmc(tbxxMap.get("KMMC") != null ? tbxxMap.get("KMMC").toString() : "");
                tbxx.setXkzbh(tbxxMap.get("KMXKZ") != null ? tbxxMap.get("KMXKZ").toString() : "");
                tbxx.setKqjb(tbxxMap.get("KQJB") != null ? tbxxMap.get("KQJB").toString() : "");
                tbxx.setSheng(tbxxMap.get("SHENG") != null ? tbxxMap.get("SHENG").toString() : "");
                tbxx.setShi(tbxxMap.get("SHI") != null ? tbxxMap.get("SHI").toString() : "");
                tbxx.setXian(tbxxMap.get("XIAN") != null ? tbxxMap.get("XIAN").toString() : "");
                tbxx.setXdm(tbxxMap.get("XDM") != null ? tbxxMap.get("XDM").toString() : "");
                tbxx.setXxdz(tbxxMap.get("KMDZ") != null ? tbxxMap.get("KMDZ").toString() : "");
                tbxx.setJjlx(tbxxMap.get("KMJJLX") != null ? tbxxMap.get("KMJJLX").toString() : "");
                tbxx.setKzlx(tbxxMap.get("KMLX") != null ? tbxxMap.get("KMLX").toString() : "");
                tbxx.setKcqtkz(tbxxMap.get("KMQTKZ") != null ? tbxxMap.get("KMQTKZ").toString() : "");
                tbxx.setKcfs(tbxxMap.get("KMKCFS") != null ? tbxxMap.get("KMKCFS").toString() : "");
                tbxx.setKczt(tbxxMap.get("KMKCZT") != null ? tbxxMap.get("KMKCZT").toString() : "");
                tbxx.setZdmj(tbxxMap.get("KMZDMJ") != null ? tbxxMap.get("KMZDMJ").toString() : "");
                tbxx.setWfmj(tbxxMap.get("KMWFMJ") != null ? tbxxMap.get("KMWFMJ").toString() : "");
                tbxx.setX(tbxxMap.get("KMX") != null ? tbxxMap.get("KMX").toString() : "");
                tbxx.setY(tbxxMap.get("KMY") != null ? tbxxMap.get("KMY").toString() : "");
                tbxx.setCzwt(tbxxMap.get("KMCZWT") != null ? tbxxMap.get("KMCZWT").toString() : "");
                tbxx.setSqtbbh(tbxxMap.get("SQTBBH") != null ? tbxxMap.get("SQTBBH").toString() : "");
                tbxx.setBxftbbh(tbxxMap.get("BFTBBH") != null ? tbxxMap.get("BFTBBH").toString() : "");
                tbxx.setBqsjsj(tbxxMap.get("BQSJSJ") != null ? tbxxMap.get("BQSJSJ").toString() : "");
                tbxx.setSqsjsj(tbxxMap.get("SQSJSJ") != null ? tbxxMap.get("SQSJSJ").toString() : "");
                tbxx.setLsks(tbxxMap.get("LKBH") != null ? tbxxMap.get("LKBH").toString() : "");
                tbxx.setZdkqmc(tbxxMap.get("ZDKQMC") != null ? tbxxMap.get("ZDKQMC").toString() : "");
                tbxx.setZrbhqmc(tbxxMap.get("BHQMC") != null ? tbxxMap.get("BHQMC").toString() : "");
                tbxx.setGyhpmc(tbxxMap.get("GYHPMC") != null ? tbxxMap.get("GYHPMC").toString() : "");
                tbxx.setGyhp03(tbxxMap.get("GYHP03") != null ? tbxxMap.get("GYHP03").toString() : "");
                tbxx.setGyhp10(tbxxMap.get("GYHP10") != null ? tbxxMap.get("GYHP10").toString() : "");
                tbxx.setCjgzmc(tbxxMap.get("CJGZMC") != null ? tbxxMap.get("CJGZMC").toString() : "");
                tbxx.setCjgz03(tbxxMap.get("CJGZ03") != null ? tbxxMap.get("CJGZ03").toString() : "");
                tbxx.setCjgzl10(tbxxMap.get("CJGZ10") != null ? tbxxMap.get("CJGZ10").toString() : "");
                tbxx.setFjmsmc(tbxxMap.get("FJMSMC") != null ? tbxxMap.get("FJMSMC").toString() : "");
                tbxx.setGjgymc(tbxxMap.get("GJGYMC") != null ? tbxxMap.get("GJGYMC").toString() : "");
                tbxx.setYsydmc(tbxxMap.get("YSYDMC") != null ? tbxxMap.get("YSYDMC").toString() : "");
                tbxx.setBz(tbxxMap.get("KFBZ") != null ? tbxxMap.get("KFBZ").toString() : "");
                tbxx.setXsly(tbxxMap.get("XSLY") != null ? tbxxMap.get("XSLY").toString() : "");
                // 导入图斑信息
                // 判断图斑数据是否已经存在
                LambdaQueryWrapper<Tbxx> queryWrapper = new LambdaQueryWrapper<>();
                queryWrapper.eq(Tbxx::getTbbh, tbxx.getTbbh());
                if (tbxxService.count(queryWrapper) == 0) {
                    taskService.importTaskByTbxx(tbxx, rwlx);
                } else {
                    failTbbhList.add(tbxx.getTbbh());
                }
            }
            // 统计导入结果
            int total = importData.size();
            int fail = failTbbhList.size();
            int success = total - fail;
            Map<String, Object> res = new HashMap<String, Object>();
            res.put("total", total);
            res.put("successCount", success);
            res.put("failCount", fail);
            res.put("failTbbhList", failTbbhList);
            return success(res);
        } catch (IOException e) {
            // 处理异常
            throw new IOException("文件上传失败");
        }
    }

    /**
     * 下发任务
     *
     * @param ids
     * @return
     */
    @ApiOperation("下发任务")
    @PreAuthorize("@ss.hasPermi('cxxm:task:distribute')")
    @Log(title = "任务", businessType = BusinessType.UPDATE)
    @PutMapping("/distribute/{ids}")
    public AjaxResult distribute(@PathVariable Long[] ids) {
        List<Task> taskList = new ArrayList<>();
        for (Long id : ids) {
            Task task = new Task();
            task.setId(id);
            task.setXfqk(XFQK.YXF.getValue());
            task.setXfsj(DateUtils.getNowDate());
            taskList.add(task);
        }
        return toAjax(taskService.updateBatchById(taskList));
    }

    /**
     * 查询任务巡查情况列表
     */
    @PreAuthorize("@ss.hasPermi('cxxm:task:list')")
    @GetMapping("/distributed/list")
    public TableDataInfo distributedList(TaskQuery task, PageDomain pageDomain) {
        startPage();
        LambdaQueryWrapper<Task> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(Objects.nonNull(task.getTbbh()), Task::getTbbh, task.getTbbh());
        queryWrapper.eq(Task::getXfqk, XFQK.YXF.getValue());
        queryWrapper.eq(Objects.nonNull(task.getRwlx()), Task::getRwlx, task.getRwlx());
        queryWrapper.in(Objects.nonNull(task.getDeptId()), Task::getDeptId, deptMapper.getAllAncestorsByDeptId(task.getDeptId()));
        List<String> drsjQuery = task.getDrsjQuery();
        if (drsjQuery != null && drsjQuery.size() == 2) {
            queryWrapper.apply(StringUtils.format("DATE_FORMAT(drsj, '%Y-%m') between '{}' and '{}'",
                    drsjQuery.get(0), drsjQuery.get(1)));
        }
        queryWrapper.orderByAsc(Task::getXfqk);
        queryWrapper.orderByDesc(Task::getXfsj);
        queryWrapper.orderByDesc(Task::getDrsj);
        List<Task> list = taskService.list(queryWrapper);
        return getDataTable(list);
    }
}

