package com.ruoyi.framework.handler;

import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.reflection.MetaObject;
import org.springframework.stereotype.Component;

/**
 * 自动填充处理类
 *
 */
@Slf4j
@Component
public class AutoFillMetaObjectHandler implements MetaObjectHandler {

    @Override
    public void insertFill(MetaObject metaObject) {
        log.info("start insert fill ....");
        // 获取当前登录用户
        String userName = SecurityUtils.getUsername();
        Long deptId = SecurityUtils.getDeptId();
        fillValue("createBy", userName, metaObject);
        fillValue("createTime", DateUtils.getNowDate(), metaObject);
        fillValue("deptId", deptId, metaObject);
    }

    @Override
    public void updateFill(MetaObject metaObject) {
        log.info("start update fill ....");
        // 获取当前登录用户
        String userName = SecurityUtils.getUsername();
        fillValue("updateBy", userName, metaObject);
        fillValue("updateTime", DateUtils.getNowDate(), metaObject);
    }

    private void fillValue(String fieldName, Object data, MetaObject metaObject) {
        if (metaObject.hasSetter(fieldName)) {
            // 值为空时设置默认值
            Object sidObj = getFieldValByName(fieldName, metaObject);
            if (sidObj == null) {
                setFieldValByName(fieldName, data, metaObject);
            }
        }
    }
}

