package com.ruoyi.framework.interceptor;

import com.baomidou.mybatisplus.core.toolkit.PluginUtils;
import com.baomidou.mybatisplus.extension.parser.JsqlParserSupport;
import com.baomidou.mybatisplus.extension.plugins.inner.InnerInterceptor;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.security.context.PermissionContextHolder;
import lombok.extern.slf4j.Slf4j;
import net.sf.jsqlparser.expression.Alias;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.expression.HexValue;
import net.sf.jsqlparser.expression.Parenthesis;
import net.sf.jsqlparser.expression.operators.conditional.AndExpression;
import net.sf.jsqlparser.parser.CCJSqlParserUtil;
import net.sf.jsqlparser.schema.Table;
import net.sf.jsqlparser.statement.delete.Delete;
import net.sf.jsqlparser.statement.select.PlainSelect;
import net.sf.jsqlparser.statement.select.Select;
import net.sf.jsqlparser.statement.select.SelectBody;
import net.sf.jsqlparser.statement.select.SetOperationList;
import net.sf.jsqlparser.statement.update.Update;
import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.executor.statement.StatementHandler;
import org.apache.ibatis.jdbc.SQL;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.session.RowBounds;

import java.lang.reflect.Method;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import static com.ruoyi.framework.aspectj.DataScopeAspect.*;

@Slf4j
public class DataScopeIntercept extends JsqlParserSupport implements InnerInterceptor {

    /**
     * 主要处理查询
     */
    @Override
    public void beforeQuery(Executor executor, MappedStatement ms, Object parameter, RowBounds rowBounds, ResultHandler resultHandler, BoundSql boundSql) throws SQLException {
        String mapperId = ms.getId();
        //获取mapper名称
        String className = mapperId.substring(0, mapperId.lastIndexOf("."));
        //获取方法名
        String methodName = mapperId.substring(mapperId.lastIndexOf(".") + 1);
        try {
            Method[] methods = Class.forName(className).getMethods();
            //遍历判断mapper 的所以方法，判断方法上是否有 UserDataScope
            for (Method m : methods) {
                if (Objects.equals(m.getName(), methodName)) {
                    DataScope controllerDataScope = m.getAnnotation(DataScope.class);
                    if (controllerDataScope != null) {
                        String originalSql = boundSql.getSql();
                        // 检查SQL是否包含count()函数
                        if (!originalSql.toLowerCase().contains("count(")) {
                            // 如果不包含count()函数，则进行SQL解析和修改
                            PluginUtils.MPBoundSql mp = PluginUtils.mpBoundSql(boundSql);
                            mp.sql(parserSingle(mp.sql(), ms.getId()));
                        }
                        // 如果包含count()函数，则不进行任何操作，直接放行
                    }
                }
            }
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 操作前置处理，可以在这里改改sql啥的
     */
    @Override
    public void beforePrepare(StatementHandler sh, Connection connection, Integer transactionTimeout) {
        InnerInterceptor.super.beforePrepare(sh, connection, transactionTimeout);
    }

    @Override
    protected void processDelete(Delete delete, int index, String sql, Object obj) {
        super.processDelete(delete, index, sql, obj);
    }

    @Override
    protected void processUpdate(Update update, int index, String sql, Object obj) {
        super.processUpdate(update, index, sql, obj);
    }

    @Override
    protected void processSelect(Select select, int index, String sql, Object obj) {
        SelectBody selectBody = select.getSelectBody();
        try {
            // 单个sql
            if (selectBody instanceof PlainSelect) {
                this.setWhere((PlainSelect) selectBody, obj.toString());
            } else if (selectBody instanceof SetOperationList) {
                // 多个sql，用;号隔开，一般不会用到。例如：select * from user;select * from role;
                SetOperationList setOperationList = (SetOperationList) selectBody;
                List<SelectBody> selects = setOperationList.getSelects();
                selects.forEach(s -> this.setWhere((PlainSelect) s, obj.toString()));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected void setWhere(PlainSelect plainSelect, String mapperId) {
        Expression sqlSegment = getSqlSegment(plainSelect, mapperId);
        if (null != sqlSegment) {
            plainSelect.setWhere(sqlSegment);
        }
    }

    private Expression getSqlSegment(PlainSelect plainSelect, String mapperId) {
        // 待执行 SQL Where 条件表达式
        Expression where = plainSelect.getWhere();
        log.info("开始进行权限过滤,where: {},mapperId: {}", where, mapperId);
        //获取mapper名称
        String className = mapperId.substring(0, mapperId.lastIndexOf("."));
        //获取方法名
        String methodName = mapperId.substring(mapperId.lastIndexOf(".") + 1);
        // 有别名用别名，无别名用表名，防止字段冲突报错
        // Table fromItem = (Table) plainSelect.getFromItem();
        // Alias fromItemAlias = fromItem.getAlias();
        // String mainTableName = fromItemAlias == null ? fromItem.getName() : fromItemAlias.getName();
        //获取当前mapper 的方法
        try {
            Method[] methods = Class.forName(className).getMethods();
            //遍历判断mapper 的所以方法，判断方法上是否有 UserDataScope
            for (Method m : methods) {
                if (Objects.equals(m.getName(), methodName)) {
                    DataScope controllerDataScope = m.getAnnotation(DataScope.class);
                    if (controllerDataScope == null) {
                        return where;
                    }
                    // 获取当前的用户
                    LoginUser loginUser = SecurityUtils.getLoginUser();
                    if (StringUtils.isNotNull(loginUser)) {
                        SysUser currentUser = loginUser.getUser();
                        // 如果是超级管理员，则不过滤数据
                        if (StringUtils.isNotNull(currentUser) && !currentUser.isAdmin()) {
                            String permission = StringUtils.defaultIfEmpty(controllerDataScope.permission(), PermissionContextHolder.getContext());
                            // 构建查询条件
                            String sql = buildDataFilter(currentUser, controllerDataScope.deptAlias(), controllerDataScope.userAlias(), permission);
                            if ("".equals(sql)) {
                                return where;
                            }
                            try {
                                Expression expression = CCJSqlParserUtil.parseExpression(sql);
                                // 数据权限使用单独的括号 防止与其他条件冲突
                                // Parenthesis parenthesis = new Parenthesis(expression);
                                if (null != where) {
                                    return new AndExpression(where, expression);
                                } else {
                                    return expression;
                                }
                            } catch (Exception e) {
                                throw new RuntimeException("数据权限解析异常 => " + e.getMessage());
                            }
                        }
                    }
                }
            }
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
        return where;
    }

    /**
     * 数据范围过滤
     *
     * @param user       用户
     * @param deptAlias  部门别名
     * @param userAlias  用户别名
     * @param permission 权限字符
     */
    public String buildDataFilter(SysUser user, String deptAlias, String userAlias, String permission) {
        StringBuilder sqlString = new StringBuilder();
        List<String> conditions = new ArrayList<String>();

        for (SysRole role : user.getRoles()) {
            String dataScope = role.getDataScope();
            if (!DATA_SCOPE_CUSTOM.equals(dataScope) && conditions.contains(dataScope)) {
                continue;
            }
            if (StringUtils.isNotEmpty(permission) && StringUtils.isNotEmpty(role.getPermissions())
                    && !StringUtils.containsAny(role.getPermissions(), Convert.toStrArray(permission))) {
                continue;
            }
            if (DATA_SCOPE_ALL.equals(dataScope)) {
                sqlString = new StringBuilder();
                conditions.add(dataScope);
                break;
            } else if (DATA_SCOPE_CUSTOM.equals(dataScope)) {
                sqlString.append(StringUtils.format(
                        " {}.dept_id IN ( SELECT dept_id FROM sys_role_dept WHERE role_id = {} ) ", deptAlias,
                        role.getRoleId()));
            } else if (DATA_SCOPE_DEPT.equals(dataScope)) {
                sqlString.append(StringUtils.format(" {}.dept_id = {} ", deptAlias, user.getDeptId()));
            } else if (DATA_SCOPE_DEPT_AND_CHILD.equals(dataScope)) {
                sqlString.append(StringUtils.format(
                        " {}.dept_id IN ( SELECT dept_id FROM sys_dept WHERE dept_id = {} or find_in_set( {} , ancestors ) )",
                        deptAlias, user.getDeptId(), user.getDeptId()));
            } else if (DATA_SCOPE_SELF.equals(dataScope)) {
                if (StringUtils.isNotBlank(userAlias)) {
                    sqlString.append(StringUtils.format(" {}.user_id = {} ", userAlias, user.getUserId()));
                } else {
                    // 数据权限为仅本人且没有userAlias别名不查询任何数据
                    sqlString.append(StringUtils.format(" {}.dept_id = 0 ", deptAlias));
                }
            }
            conditions.add(dataScope);
        }

        // 多角色情况下，所有角色都不包含传递过来的权限字符，这个时候sqlString也会为空，所以要限制一下,不查询任何数据
        if (StringUtils.isEmpty(conditions)) {
            sqlString.append(StringUtils.format(" {}.dept_id = 0 ", deptAlias));
        }
        return sqlString.toString();
    }


}

